/* client.c -	 client program that uses the socket interface to tcp
 *		 This client is modelled on rping -c.
 *		 It accepts a number of command line options,
 *		 then repeatedly sends messages to the server,
 *		 and reads back the replies
 *		 Modified for exs API.
 *  on Solaris this program must be linked with -lnsl and -lsocket
 */

/*
 * Copyright 2007, 2008, 2009, 2010, 2011, 2012 by
 *	University of New Hampshire Interoperability Laboratory
 *	121 Technology Drive, Suite 2
 *	Durham, NH 03824-4716
 *	phone: 1-603-862-0090
 *	fax: 1-603-862-4181
 *	e-mail: info@iol.unh.edu
 * All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grant No. OCI-1127228.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#define _POSIX_C_SOURCE 200112L
#define _ISOC99_SOURCE

#if defined(__sun__)
	#define __EXTENSIONS__
#endif

#define _BSD_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <ctype.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <fcntl.h>
#include <time.h>
#include <netdb.h>

#include "utilities.h"
#include "common.h"


int
main(int argc, char *argv[])
{
	int			fd, i, n, mess_size, count1;

	/* sizeof(struct sockaddr_in)	is 16
	 * sizeof(struct sockaddr_in6)	is 28
	 * sizeof(struct sockaddr)	is 16 !!! should be 28 !!!
	 */
	char			text_buf[TEXT_SIZE];
	char			*ptr, c, first_char = 'A';
	char			*in_ptr;
	exs_mhandle_t		in_mhandle;
	exs_ahandle_t		in_ahandle;
	exs_event_t		completion;
	struct rusage		start_usage;
	int			secs, usecs;
	int			user_secs, user_usecs;
	int			kernel_secs, kernel_usecs;
	int			len;


	startup(argc, argv, 1);

	/* establish a connection to indicated server */
	LATENEXS_TRACE(stderr, "%s client: calling openclient to connect to "
			"server\n", time_prefix());
	fd = openclient(server_port, server_name,
			SOCK_SEQPACKET,
			(struct sockaddr *)&server_addr,
			(struct sockaddr *)&client_addr,
			&our_conn_params);
	LATENEXS_TRACE(stderr, "%s client: openclient returned fd %d\n",
			time_prefix(), fd);
	if (fd < 0)
		exit(EXIT_FAILURE);

	/* we are now successfully connected to a remote server */
	print_ip_address((struct sockaddr *)&client_addr, "client", "latenexs");
	print_ip_address((struct sockaddr *)&server_addr, "server", "latenexs");
	print_attributes(fd, "client");


	fprintf(stderr,"%s client: give server 1 second to set up\n",
		time_prefix());
	sleep(1);

	/* send a block of "size" bytes of printable data to server
	 * do this "count" times
	 * for each send, we should get back an identical reply
	 */
	c = ' ';
	ptr = out_mess;
	for (n = 0; n < size; n++) {
		if (!isprint(c))
			c = ' ';
		*ptr++ = c++;
	}
	*ptr = '\0';
	mess_size = size;

	fprintf(stderr, "client writing %d-byte buffers %d times\n",
								size, count);

	count1 = count - 1;

	/* send a "go" to server to say we're ready */
	strcpy(text_buf, "go");
	fprintf(stderr, "client: writeblock \"%s\"\n", text_buf);
	len = writeblock(fd, text_buf, 3, 0, EXS_MHANDLE_UNREGISTERED, send_queue);
	if (len <= 0) {
		fprintf(stderr, "client: writeblock %s\n",
			strerror(errno));
		exit(EXIT_FAILURE);
	}

	/* wait for server to tell us "ok" */
	len = readpacket(fd, text_buf, TEXT_SIZE,
					EXS_MHANDLE_UNREGISTERED, recv_queue);
	if (len <= 0) {
		fprintf(stderr, "client: readpacket %s\n",
			strerror(errno));
		exit(EXIT_FAILURE);
	}
	fprintf(stderr, "client: readpacket \"%s\"\n", text_buf);

	LATENEXS_TRACE(stderr,"\n%s client: starting first iteration %d\n\n",
			time_prefix(), 0);

	/* start the timer immediately before first transaction starts */
	if( getrusage(RUSAGE_SELF, &start_usage) < 0 ) {
		printf("client: getrusage start %s\n", strerror(errno));
		exit(EXIT_FAILURE);
		}
	if( gettimeofday(&start_time, NULL) < 0) {
		printf("%s client gettimeofday start %s\n", time_prefix(),
			strerror(errno));
		exit(EXIT_FAILURE);
		}

	/* make first char in each buffer different */
	ptr = out_mess;
	(*ptr)++;
	if (!isprint(*ptr))
		*ptr = ' ';

		LATENEXS_TRACE(stderr, "%s client: calling first exs_recv, fd %d, "
				"maxlen %d, in_mess %p, in_handle %p, "
				"recv_queue %p\n", time_prefix(), fd,
				mess_size, in_mess, in_handle,
				recv_queue);

	/* set up the first "pong" receive */
	if (exs_recv(fd, in_mess, mess_size, recv_flags,
				recv_queue, NULL, in_handle) < 0) {
		fprintf(stderr, "%s client pong exs_recv: %s\n",
					time_prefix(), strerror(errno));
		goto err2;
	}

	for (i = 1; i < count; i++) {
		out_mess[size] = '\0';
		LATENEXS_TRACE(stderr, "%s client: calling exs_send, fd %d, "
				"message len %d, message address %p, "
				"out_handle %p, message: \"%.40s\"\n",
				time_prefix(), fd, size,
				out_mess, out_handle, out_mess);

		/* send "ping" message without completion event back */
		if (exs_send(fd, out_mess, mess_size,
					send_flags | EXS_UNSIGNALED,
					NULL, NULL, out_handle) < 0) {
			fprintf(stderr, "%s client exs_send: %s\n",
					time_prefix(), strerror(errno));
			goto err2;
		}

		/* wait for the "pong" reply from the server */
		if (exs_qdequeue(recv_queue, &completion, 1, NULL) < 0) {
			fprintf(stderr, "%s client recv exs_qdequeue: %s\n",
						time_prefix(), strerror(errno));
			break;
		}

		in_ptr = completion.exs_evt_union.exs_evt_xfer.exs_evt_buffer;
		in_mhandle
			= completion.exs_evt_union.exs_evt_xfer.exs_evt_mhandle;

		LATENEXS_TRACE(stderr, "%s client: pong exs_recv complete, fd "
				"%d, in_ptr %p ?=? in_mess %p, in_mhandle %p "
				"?=? in_handle %p\n", time_prefix(), fd,
				in_ptr, in_mess, in_mhandle, in_handle);

#if defined(ALLOW_FULL_CHECKING)
		if (completion.exs_evt_errno != 0) {
			fprintf(stderr, "%s client pong recv completion: %s\n",
				time_prefix(),
				strerror(completion.exs_evt_errno));
			break;
		}

		len = completion.exs_evt_union.exs_evt_xfer.exs_evt_length;

		if (len <= 0) {
			break;
		}

		LATENEXS_TRACE(stderr, "%s client: pong exs_recv "
				"returned %d bytes\n",
				time_prefix(), len);

		if (len != size) {
			fprintf(stderr, "%s client: in_mess was %d, "
				"expected %d\n",
				time_prefix(), len, size);
		}

#if defined(ALLOW_DEBUG)
		if (laten_debug_flag) {
			in_mess[n] = '\0';/* make data into a C-string */
			fprintf(stderr, "%s client: got %d char string from "
					"server: \"%.40s\"\n", time_prefix(), n,
					in_ptr);
		}
#endif

#endif

		LATENEXS_TRACE(stderr,"\n%s client: starting iteration %d\n\n",
				time_prefix(), i);

		/* make first char in each buffer different */
		ptr = out_mess;
		*ptr = first_char++;
		if (!isprint(first_char))
			first_char = 'A';

		LATENEXS_TRACE(stderr, "%s client: calling exs_recv, fd %d, "
				"maxlen %d, in_mess %p, in_handle %p, "
				"recv_queue %p\n", time_prefix(), fd,
				mess_size, in_mess, in_handle,
				recv_queue);

		if (i == count1) {
			LATENEXS_TRACE(stderr, "%s client: finished %d iterations, "
				"set up recv of final pong\n", time_prefix(),i);
		}
		/* set up the next "pong" receive */
		if (exs_recv(fd, in_mess, mess_size, recv_flags,
					recv_queue, NULL, in_handle) < 0) {
			fprintf(stderr, "%s client pong exs_recv: %s\n",
						time_prefix(), strerror(errno));
			break;
		}

	}	/* for loop */

	LATENEXS_TRACE(stderr, "%s client final exs_send, iteration %d\n",
							time_prefix(), i);

	/* send final "ping" message without completion event back */
	if (exs_send(fd, out_mess, mess_size, send_flags | EXS_UNSIGNALED,
				NULL, NULL, out_handle) < 0) {
		fprintf(stderr, "%s client exs_send: %s\n",
				time_prefix(), strerror(errno));
		goto err2;
	}

	/* wait for the final "pong" reply from the server */
	if (exs_qdequeue(recv_queue, &completion, 1, NULL) < 0) {
		fprintf(stderr, "%s client final recv exs_qdequeue: %s\n",
					time_prefix(), strerror(errno));
		goto err1;
	}

	in_ptr = completion.exs_evt_union.exs_evt_xfer.exs_evt_buffer;
	in_mhandle = completion.exs_evt_union.exs_evt_xfer.exs_evt_mhandle;
	in_ahandle = completion.exs_evt_ahandle;

	LATENEXS_TRACE(stderr, "%s client: final exs_recv, fd %d, "
			"in_ptr %p ?=? in_mess %p, in_mhandle %p ?=? "
			"in_handle %p, in_ahandle %p ?=? &fd %p\n",
			time_prefix(), fd,
			in_ptr, in_mess, in_mhandle, in_handle,
			in_ahandle, &fd);

	LATENEXS_TRACE(stderr,
			"%s client final exs_recv completed, stop timers\n",
			time_prefix());

	if (completion.exs_evt_errno != 0) {
		fprintf(stderr, "%s client final recv completion: %s\n",
			time_prefix(),
			strerror(completion.exs_evt_errno));
		goto err1;
	}

	len = completion.exs_evt_union.exs_evt_xfer.exs_evt_length;

	if (len <= 0) {
		goto err1;
	}

	LATENEXS_TRACE(stderr, "%s client: final exs_recv returned"
			" %d bytes\n",
			time_prefix(), len);

	if (n != size) {
		fprintf(stderr, "%s client: final in_mess was %d, "
			"expected %d\n",
			time_prefix(), len, size);
	}

#if defined(ALLOW_DEBUG)
	if (laten_debug_flag) {
		in_mess[n] = '\0';/* make data into a C-string */
		fprintf(stderr, "%s client: got %d char string from "
				"server: \"%.40s\"\n", time_prefix(), n,
				in_ptr);
	}
#endif


	if (in_ptr != in_mess) {
		fprintf(stderr, "final in_ptr %p != out_mess %p\n",
						in_ptr, out_mess);
	}

err1:
	LATENEXS_TRACE(stderr, "%s client: stop timers\n", time_prefix());

	/* stop the timer immediately after last transaction completes */
	get_relative_time(&secs, &usecs);
	get_relative_usage(&start_usage, &user_secs, &user_usecs,
                                                &kernel_secs, &kernel_usecs);

	LATENEXS_TRACE(stderr, "\n");

	print_latency("client", i, secs, usecs);
	fprintf(stderr, "client: %11d.%06d seconds elapsed\n", secs, usecs);
	fprintf(stderr, "client: %11d.%06d seconds user (%.4f%%)\n",
		user_secs, user_usecs,
		percentage(secs, usecs, user_secs, user_usecs));
	fprintf(stderr, "client: %11d.%06d seconds kernel (%.4f%%)\n",
		kernel_secs, kernel_usecs,
		percentage(secs, usecs, kernel_secs, kernel_usecs));

err2:
	if (use_registered_memory) {
		/* deregister the memory */
		LATENEXS_TRACE(stderr, "client: deregister in_handle %p\n",
								in_handle);
		if (exs_mderegister(in_handle, 0)) {
			perror("client deregister in handle");
		}
		LATENEXS_TRACE(stderr, "client: deregister out_handle %p\n",
								out_handle);
		if (exs_mderegister(out_handle, 0)) {
			perror("client deregister out handle");
		}
	}

	/* close the connection to the server */
	LATENEXS_TRACE(stderr, "client: close connection fd = %d\n", fd);

	if (exs_close(fd, 0, recv_queue, NULL) < 0) {
		perror("client exs_close");
	} else {
		LATENEXS_TRACE(stderr, "client: wait for exs_close %d\n", fd);
		if (asynch_wait(recv_queue, &completion, NULL) < 0) {
			perror("client exs_close asynch_wait");
		}
	}

	LATENEXS_TRACE(stderr, "client: delete send_queue %p\n", send_queue);
	if (exs_qdelete(send_queue) < 0) {
		perror("client exs_qdelete send_queue");
	}

	LATENEXS_TRACE(stderr, "client: delete recv_queue %p\n", recv_queue);
	if (exs_qdelete(recv_queue) < 0) {
		perror("client exs_qdelete recv_queue");
	}

	LATENEXS_TRACE(stderr, "client: exit\n");
	exit(EXIT_SUCCESS);
}

/* vi: set autoindent tabstop=8 shiftwidth=8 : */
