/* utilities.h */

/* Modified for exs API */

/* On Solaris this program must be linked with -lnsl and -lsocket */

/*
 * Copyright 2007, 2008, 2009, 2010, 2011, 2012 by
 *	University of New Hampshire Interoperability Laboratory
 *	121 Technology Drive, Suite 2
 *	Durham, NH 03824-4716
 *	phone: 1-603-862-0090
 *	fax: 1-603-862-4181
 *	e-mail: info@iol.unh.edu
 * All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grant No. OCI-1127228.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#ifndef UTILITIES_H
#define UTILITIES_H

#include <sys/socket.h>
#include <exs.h>

/* EXS connection parameters. Value parameters may be left at their default by
 * using a value of INT_MAX here. */
struct conn_params {
	/* all fd flags for fcntl OR'd together */
	int fd_flags;

	/* Small packet size for Nagle */
	int small_packet_max_size;

	/* Maximum amount of inline data to attempt to use */
	int max_inline;

	/* CPU to bind completion thread to */
	int comp_cpu;

	/* Maximum number of flow control credits */
	int max_credits;
};

extern void no_sigpipe(void);

extern void fstatdump( int limit );

void blockio_set_debug(int flag);

/* Returns after starting write of nbytes characters from buffer to fd.  */
int
asynch_send(int fd, char *buffer, int nbytes, int flags,
		exs_mhandle_t buf_handle, exs_qhandle_t wait_queue);

/* Returns after starting write of count vectors from sendvec to fd.  */
int
asynch_sendfile(int fd, exs_xferfile_t *sendvec, int count,
		exs_qhandle_t wait_queue, exs_ahandle_t ahandle);

/* Returns after starting read of nbytes characters into buffer from fd.  */
int
asynch_recv(int fd, char *buffer, int nbytes, int flags,
		exs_qhandle_t wait_queue,
		exs_ahandle_t ahandle,
		exs_mhandle_t buf_handle);

/* Returns after getting an async event from wait_queue	*/
int
asynch_wait(exs_qhandle_t wait_queue, exs_event_t *vector,
		struct timeval *timeout);

int writeblock(int fd, char *buffer, int nbytes, int flags,
		exs_mhandle_t buf_handle,
		exs_qhandle_t wait_queue);

int readblock(int fd, char *buffer, int nbytes, exs_qhandle_t wait_queue);

/* Returns after <= nbytes characters have been read into buffer from fd.
 * Will also return if an end of file is encountered.
 * Value returned is actual number of bytes read.
 */
int readpacket(int fd, char *buffer, int nbytes,
		exs_mhandle_t buf_handle,
		exs_qhandle_t wait_queue);

int print_attributes(int fd, char *message);

/* convert and print IP address as dotted-decimal or colon-hex string */
void
print_ip_address(struct sockaddr *addr, char *who, char *message);

int openclient(char *server_port, char *server_node, int socktype,
		struct sockaddr *server_addr,
		struct sockaddr *client_addr,
		const struct conn_params *params);

int openserver(char *server_port, char *server_name, int listening_depth,
		int socktype, struct sockaddr *server_address,
		const struct conn_params *params);

#endif

/* vi: set autoindent tabstop=8 shiftwidth=8 : */
