#	atomic.s

# Copyright 2007, 2008, 2009, 2010, 2011, 2012 by
#	University of New Hampshire Interoperability Laboratory
#	121 Technology Drive, Suite 2
#	Durham, NH 03824-4716
#	phone: 1-603-862-0090
#	fax: 1-603-862-4181
#	e-mail: info@iol.unh.edu
# All rights reserved.
#
# This file is part of the UNH EXS implementation of the
# Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
# Document Number C050 published by The Open Group, January 2005,
# and copyright 2005 by The Open Group.
#
# This software is available to you under a choice of one of two
# licenses.  You may choose to be licensed under the terms of the GNU
# General Public License (GPL) Version 2, available from the file
# COPYING.GPL in the main directory of this source tree, or the
# BSD license, available from the file COPYING.BSD in the main directory
# of this source tree.
#
# This material is based upon work supported by the National Science
# Foundation under Grant No. OCI-1127228.
#
# Any opinions, findings, and conclusions or recommendations
# expressed in this material are those of the authors and do not
# necessarily reflect the views of the National Science Foundation.

#
#	unsigned int atomic_add_reset(
#				unsigned int increment,
#				unsigned int limit,
#				unsigned  int *result)
#
#	atomically updates the value in *result to
#	*result + increment	if that is < limit
#	0			otherwise
#
	.text
	.align	4
	.globl	atomic_add_reset

atomic_add_reset:
	pushl	%ebx		# free up a work register
	movl	16(%esp),%ecx	# get ptr to result in ecx
	movl	8(%esp),%ebx	# get increment into ebx
	movl	(%ecx),%eax	# get old result into eax for 1st time thru loop
1:
	leal	(%eax,%ebx),%edx# get current result + increment into edx
	cmpl	%edx,12(%esp)	# compare result + increment in edx with limit
	cmovbel	zero,%edx	# reset edx to 0 if result + increment >= limit
	lock			# makes the cmpxchgl atomic
	cmpxchgl %edx,(%ecx)	# change result to edx if current result == eax
	jnz	1b		# loop if result changed by another task, and
				# thus eax has been reloaded with that result
#	fall thru if new result = old result + increment
	movl	%edx,%eax	# return new result in eax
	popl	%ebx		# restore borrowed register
	ret			# return what the caller produced in eax

zero:	.long	0		# constant for reseting result
#
# the cmpxchgl src,dest works atomically as follows:
#	eax is implicitly referenced and is expected to hold old dest value
#	src must be a register and should hold the new dest value
#	dest can be memory or a register
# the idea is to replace old dest value with new dest value IF dest is unchanged
#
# when eax == dest then			# old dest value unchanged since eax
#	ZF = 1; dest = src		# so replace old dest with new dest
#					# old dest value remains in eax
# otherwise when eax != dest then	# old dest value changed since eax
#	ZF = 0; eax = dest		# get current (changed) dest into eax
#
# the cmovccl src,dest works as follows:
#	when the EFLAGS match the cc conditions (as in the Jcc instruction)
#	then the move takes place, else it doesn't
# in the EFLAGS condition codes:
#	"above" and "below" refer to the relation between unsigned integers
#	"greater" and "less" refer to the relation between signed integers
# Note that the following common instructions do NOT change the EFLAGS:
#	lea, mov, push, pop

	.text
	.align	4
	.globl	atomic_set_bits

atomic_set_bits:
	movl	8(%esp),%ecx	# get ptr to result in ecx
	movl	(%ecx),%eax	# get old result into eax
1:
	movl	%eax,%edx	# copy current result into edx
	orl	4(%esp),%edx	# or mask onto current result
	lock
	cmpxchgl %edx,(%ecx)	# try to change current result
	jnz	1b		# try again if change didn't happen, and
				# eax has been reloaded with current result
#	fall thru if new result = old result | mask
	movl	%edx,%eax	# return new result in eax
	ret			# so caller knows what he produced

	.text
	.align	4
	.globl	atomic_clear_bits

atomic_clear_bits:
	pushl	%ebx		# free up a work register
	movl	8(%esp),%ebx	# get mask into ebx
	notl	%ebx		# get ~mask into ebx
	movl	12(%esp),%ecx	# get ptr to result in ecx
	movl	(%ecx),%eax	# get old result into eax
1:
	movl	%eax,%edx	# copy current result into edx
	andl	%ebx,%edx	# and ~mask onto current result
	lock
	cmpxchgl %edx,(%ecx)	# try to change current result
	jnz	1b		# try again if change didn't happen, and
				# eax has been reloaded with current result
#	fall thru if new result = old result | mask
	movl	%edx,%eax	# return new result in eax
	popl	%ebx		# restore work register
	ret			# so caller knows what he produced
#
# the cmpxchgl src,dest works atomically as follows:
#	eax is implicitly referenced and is expected to hold old dest value
#	src must be a register and should hold the new dest value
#	dest can be memory or a register
# the idea is to replace old dest value with new dest value IF dest is unchanged
#
# when eax == dest then			# old dest value unchanged since eax
#	ZF = 1; dest = src		# so replace old dest with new dest
#					# old dest value remains in eax
# otherwise when eax != dest then	# old dest value changed since eax
#	ZF = 0; eax = dest		# get current (changed) dest into eax

#	unsigned int atomic_swap_if_match(
#				unsigned int swap_value,
#				unsigned int match_value,
#				unsigned int *result)
#
#	atomically updates the value in *result to
#	*swap_value		if match_value == match_value
#
#	equivalent to:
#		atomically:
#			unsigned int ret = *result;
#			iffff (*result == match_value) {
#				*result = swap_value;
#			}
#			return ret;
#
#		returns value of *result prior to attempted swap
#			so return value == match_value if swap was done,
#						and now *result == swap_value
#			   return value == *result if swap was not done,
#						and now *result == old result
#
	.text
	.align	4
	.globl	atomic_swap_if_match

atomic_swap_if_match:
	movl	12(%esp),%ecx	# get result address in ecx
	movl	8(%esp),%eax	# get match_value into eax
	movl	4(%esp),%edx	# get swap_value into edx
	lock			# makes the cmpxchgl atomic
	cmpxchgl %edx,(%ecx)	# try to change current result to swap_value
#
#	always fall thru
# iffff change didn't happen because %eax was not equal to (%ecx)
#	then %eax has been reloaded with old result (%ecx)
# iffff change did happen because %eax was equal to (%ecx)
#	then new result = swap_value
#	and neither %eax nor %edx were changed, only (%ecx) was changed
#	so %eax already has old result because that equaled the match_value
	ret

	.text
	.align	4
	.globl	atomic_swap_if_match_ptr

atomic_swap_if_match_ptr:
	jmp atomic_swap_if_match
