/* our_calloc_free.h */

#ifndef OUR_CALLOC_FREE_H_
#define OUR_CALLOC_FREE_H_

/*
 * Copyright 2007, 2008, 2009, 2010, 2011, 2012 by
 *	University of New Hampshire Interoperability Laboratory
 *	121 Technology Drive, Suite 2
 *	Durham, NH 03824-4716
 *	phone: 1-603-862-0090
 *	fax: 1-603-862-4181
 *	e-mail: info@iol.unh.edu
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grant No. OCI-1127228.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

/*
 * The UNH IOL implementation of the Extended Sockets API (ES-API) Issue 1.0
 *
 * defined in:	The Interconnect Software Consortium Issue 1.0
 *		ISBN: 1-931624-52-6
 *		Published by The open Group, January 2005.
 *
 * all page references in this source code are to pages in that document.
 */

/*
 * Most of the coding in this file done by:
 *	Bob Russell
 *	rdr@iol.unh.edu
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#if defined(ALLOW_EXS_STATISTICS)

	enum statistics_counter_type {
		completion_loops,
		good_ibv_poll_cqs,
		empty_ibv_poll_cqs,
		good_ibv_get_cq_events,
		good_completions,
		send_advert_completions,
		send_rdma_write_completions,
		recv_advert_completions,
		recv_rdma_with_imm_completions,
		recv_eof_completions,
		total_setup_completions,
		matched_recv_await,
		unmatched_recv_await,
		send_done_after_sending,
		send_done_after_completion,
		total_setup_sends,
		advert_blocks_sent,
		signaled_advert_blocks_sent,
		unsignaled_advert_blocks_sent,
		signaled_rdma_write_blocks_sent,
		unsignaled_rdma_write_blocks_sent,
		rdma_write_blocks_sent,
		rdma_write_bytes_sent,
		max_statistics_counter_type
	};

#define STATISTICS_INCREMENT(arg)	\
	do	{		\
		statistics_counters[arg]++;		\
	} while (0)

#define STATISTICS_DECREMENT(arg)	\
	do	{		\
		statistics_counters[arg]--;		\
	} while (0)

#define STATISTICS_ADD(arg, val)	\
	do	{		\
		statistics_counters[arg] += val;	\
	} while (0)

#else

#define STATISTICS_INCREMENT(arg)	\
	do	{		\
	} while (0)

#define STATISTICS_ADD(arg, val)	\
	do	{		\
	} while (0)

#endif


#if defined(ALLOW_EXS_TRACE)

extern int _our_debug_flag;

#define TRACING_IS_ON	_our_debug_flag

#define TRACE(...)						\
	do	{						\
		if(_our_debug_flag) {				\
			fprintf(__VA_ARGS__);			\
		}						\
	} while (0)

#define TRACE_VAR_DECLARE(type, name)	type name;

#define TRACE_VAR_ASSIGN(lhs, rhs)	lhs = rhs

#define TRACE_VAR_INCREMENT(var)	var ++

#define MY_NAME(string)						\
	static char	my_name[] = string;

#define PRINT_STATUS(...)					\
	do	{						\
		if(_our_debug_flag) {				\
			our_pstatus(__VA_ARGS__);		\
		}						\
	} while (0)

#define DUMP_BUF(...)						\
	do	{						\
		if(_our_debug_flag) {				\
			our_dump_buf(__VA_ARGS__);		\
		}						\
	} while (0)

#define TRACE_EVENT_TYPE(...)					\
	do	{						\
		if(_our_debug_flag) {				\
			our_trace_event_type(__VA_ARGS__);	\
		}						\
	} while (0)						\

#else

#define TRACING_IS_ON	0

#define TRACE(args...)						\
	do	{						\
	} while (0)

#define TRACE_VAR_DECLARE(type, name)

#define TRACE_VAR_ASSIGN(lhs, rhs)	do { } while (0)

#define TRACE_VAR_INCREMENT(var)	do { } while (0)

#define MY_NAME(string)

#define PRINT_STATUS(args...)					\
	do	{						\
	} while (0)

#define DUMP_BUF(args...)					\
	do	{						\
	} while (0)

#define TRACE_EVENT_TYPE(args...)				\
	do	{						\
	} while (0)

#endif


#if defined(ALLOW_EXS_CALLOC)

#define EXS_CALLOC(nmemb, size)					\
	_our_calloc_calloc(nmemb, size, __FUNCTION__, __LINE__)

#define EXS_FREE(address)					\
	_our_calloc_free(address, __LINE__)

#define EXS_CALLOC_REPORT(fptr)						\
	_our_calloc_report(fptr)

extern void *
_our_calloc_calloc(size_t nmemb, size_t size, const char * fname,
		unsigned int lineno);

extern void
_our_calloc_free( void *address, unsigned int lineno );

extern void
_our_calloc_report( FILE *fptr );

#else

#define EXS_CALLOC(nmemb, size)					\
	calloc(nmemb, size)

#define EXS_FREE(address)					\
	free(address)

#define EXS_CALLOC_REPORT(fptr)					\
	do { } while(0)

#endif


#endif /* OUR_CALLOC_FREE_H_ */

/* vi: set autoindent tabstop=8 shiftwidth=8 : */
